@@ -1,16 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
 # Copyright 2019 The meson development team
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+
+from __future__ import annotations
 
 """Abstractions for the Intel Compiler families.
 
@@ -27,6 +18,10 @@ from ... import mesonlib
 from ..compilers import CompileCheckMode
 from .gnu import GnuLikeCompiler
 from .visualstudio import VisualStudioLikeCompiler
+from ...options import OptionKey
+
+if T.TYPE_CHECKING:
+    from ...environment import Environment
 
 # XXX: avoid circular dependencies
 # TODO: this belongs in a posix compiler class
@@ -49,16 +44,13 @@ class IntelGnuLikeCompiler(GnuLikeCompiler):
     minsize: -O2
     """
 
-    BUILD_ARGS = {
-        'plain': [],
-        'debug': ["-g", "-traceback"],
-        'debugoptimized': ["-g", "-traceback"],
-        'release': [],
-        'minsize': [],
-        'custom': [],
-    }  # type: T.Dict[str, T.List[str]]
+    DEBUG_ARGS: T.Dict[bool, T.List[str]] = {
+        False: [],
+        True: ['-g', '-traceback']
+    }
 
-    OPTIM_ARGS = {
+    OPTIM_ARGS: T.Dict[str, T.List[str]] = {
+        'plain': [],
         '0': ['-O0'],
         'g': ['-O0'],
         '1': ['-O1'],
@@ -66,6 +58,7 @@ class IntelGnuLikeCompiler(GnuLikeCompiler):
         '3': ['-O3'],
         's': ['-Os'],
     }
+    id = 'intel'
 
     def __init__(self) -> None:
         super().__init__()
@@ -74,10 +67,9 @@ class IntelGnuLikeCompiler(GnuLikeCompiler):
         # It does have IPO, which serves much the same purpose as LOT, but
         # there is an unfortunate rule for using IPO (you can't control the
         # name of the output file) which break assumptions meson makes
-        self.base_options = {mesonlib.OptionKey(o) for o in [
+        self.base_options = {OptionKey(o) for o in [
             'b_pch', 'b_lundef', 'b_asneeded', 'b_pgo', 'b_coverage',
             'b_ndebug', 'b_staticpic', 'b_pie']}
-        self.id = 'intel'
         self.lang_header = 'none'
 
     def get_pch_suffix(self) -> str:
@@ -87,10 +79,10 @@ class IntelGnuLikeCompiler(GnuLikeCompiler):
         return ['-pch', '-pch_dir', os.path.join(pch_dir), '-x',
                 self.lang_header, '-include', header, '-x', 'none']
 
-    def get_pch_name(self, header_name: str) -> str:
-        return os.path.basename(header_name) + '.' + self.get_pch_suffix()
+    def get_pch_name(self, name: str) -> str:
+        return os.path.basename(name) + '.' + self.get_pch_suffix()
 
-    def openmp_flags(self) -> T.List[str]:
+    def openmp_flags(self, env: Environment) -> T.List[str]:
         if mesonlib.version_compare(self.version, '>=15.0.0'):
             return ['-qopenmp']
         else:
@@ -113,8 +105,8 @@ class IntelGnuLikeCompiler(GnuLikeCompiler):
     def get_profile_use_args(self) -> T.List[str]:
         return ['-prof-use']
 
-    def get_buildtype_args(self, buildtype: str) -> T.List[str]:
-        return self.BUILD_ARGS[buildtype]
+    def get_debug_args(self, is_debug: bool) -> T.List[str]:
+        return self.DEBUG_ARGS[is_debug]
 
     def get_optimization_args(self, optimization_level: str) -> T.List[str]:
         return self.OPTIM_ARGS[optimization_level]
@@ -127,16 +119,13 @@ class IntelVisualStudioLikeCompiler(VisualStudioLikeCompiler):
 
     """Abstractions for ICL, the Intel compiler on Windows."""
 
-    BUILD_ARGS = {
-        'plain': [],
-        'debug': ["/Zi", "/traceback"],
-        'debugoptimized': ["/Zi", "/traceback"],
-        'release': [],
-        'minsize': [],
-        'custom': [],
-    }  # type: T.Dict[str, T.List[str]]
+    DEBUG_ARGS: T.Dict[bool, T.List[str]] = {
+        False: [],
+        True: ['/Zi', '/traceback']
+    }
 
-    OPTIM_ARGS = {
+    OPTIM_ARGS: T.Dict[str, T.List[str]] = {
+        'plain': [],
         '0': ['/Od'],
         'g': ['/Od'],
         '1': ['/O1'],
@@ -145,9 +134,7 @@ class IntelVisualStudioLikeCompiler(VisualStudioLikeCompiler):
         's': ['/Os'],
     }
 
-    def __init__(self, target: str) -> None:
-        super().__init__(target)
-        self.id = 'intel-cl'
+    id = 'intel-cl'
 
     def get_compiler_check_args(self, mode: CompileCheckMode) -> T.List[str]:
         args = super().get_compiler_check_args(mode)
@@ -171,11 +158,11 @@ class IntelVisualStudioLikeCompiler(VisualStudioLikeCompiler):
         version = int(v1 + v2)
         return self._calculate_toolset_version(version)
 
-    def openmp_flags(self) -> T.List[str]:
+    def openmp_flags(self, env: Environment) -> T.List[str]:
         return ['/Qopenmp']
 
-    def get_buildtype_args(self, buildtype: str) -> T.List[str]:
-        return self.BUILD_ARGS[buildtype]
+    def get_debug_args(self, is_debug: bool) -> T.List[str]:
+        return self.DEBUG_ARGS[is_debug]
 
     def get_optimization_args(self, optimization_level: str) -> T.List[str]:
         return self.OPTIM_ARGS[optimization_level]
