@@ -5,22 +5,74 @@ from __future__ import annotations
 
 from .base import ExternalDependency, DependencyException, DependencyTypeName
 from .pkgconfig import PkgConfigDependency
-from ..mesonlib import (Popen_safe, OptionKey, join_args, version_compare)
+from ..mesonlib import (Popen_safe, join_args, version_compare, version_compare_many)
+from ..options import OptionKey
 from ..programs import ExternalProgram
 from .. import mlog
+from enum import Enum
 import re
 import os
 import json
 import typing as T
 
 if T.TYPE_CHECKING:
+    from typing_extensions import TypedDict
+
     from ..environment import Environment
 
+    # Definition of what `dub describe` returns (only the fields used by Meson)
+    class DubDescription(TypedDict):
+        platform: T.List[str]
+        architecture: T.List[str]
+        buildType: str
+        packages: T.List[DubPackDesc]
+        targets: T.List[DubTargetDesc]
+
+    class DubPackDesc(TypedDict):
+        name: str
+        version: str
+        active: bool
+        configuration: str
+        path: str
+        targetType: str
+        targetFileName: str
+
+    class DubTargetDesc(TypedDict):
+        rootPackage: str
+        linkDependencies: T.List[str]
+        buildSettings: DubBuildSettings
+        cacheArtifactPath: str
+
+    class DubBuildSettings(TypedDict):
+        importPaths: T.List[str]
+        stringImportPaths: T.List[str]
+        versions: T.List[str]
+        mainSourceFile: str
+        sourceFiles: T.List[str]
+        dflags: T.List[str]
+        libs: T.List[str]
+        lflags: T.List[str]
+
+    class FindTargetEntry(TypedDict):
+        search: str
+        artifactPath: str
+
+class DubDescriptionSource(Enum):
+    Local = 'local'
+    External = 'external'
 
 class DubDependency(ExternalDependency):
     # dub program and version
     class_dubbin: T.Optional[T.Tuple[ExternalProgram, str]] = None
     class_dubbin_searched = False
+    class_cache_dir = ''
+
+    # Map Meson Compiler ID's to Dub Compiler ID's
+    _ID_MAP: T.Mapping[str, str] = {
+        'dmd': 'dmd',
+        'gcc': 'gdc',
+        'llvm': 'ldc',
+    }
 
     def __init__(self, name: str, environment: 'Environment', kwargs: T.Dict[str, T.Any]):
         super().__init__(DependencyTypeName('dub'), environment, kwargs, language='d')
@@ -40,37 +92,36 @@ class DubDependency(ExternalDependency):
         if DubDependency.class_dubbin is None:
             if self.required:
                 raise DependencyException('DUB not found.')
-            self.is_found = False
             return
 
         (self.dubbin, dubver) = DubDependency.class_dubbin  # pylint: disable=unpacking-non-sequence
 
         assert isinstance(self.dubbin, ExternalProgram)
 
-        # Check if Dub version is compatible with Meson
-        if version_compare(dubver, '>1.31.1'):
+        # Check Dub's compatibility with Meson
+        self._search_in_cache = version_compare(dubver, '<=1.31.1')
+        self._use_cache_describe = version_compare(dubver, '>=1.35.0')
+        self._dub_has_build_deep = version_compare(dubver, '>=1.35.0')
+
+        if not self._search_in_cache and not self._use_cache_describe:
             if self.required:
                 raise DependencyException(
-                    f"DUB version {dubver} is not compatible with Meson (can't locate artifacts in Dub cache)")
-            self.is_found = False
+                    f'DUB version {dubver} is not compatible with Meson'
+                    " (can't locate artifacts in DUB's cache). Upgrade to Dub >= 1.35.")
+            else:
+                mlog.warning(f'DUB dependency {name} not found because Dub {dubver} '
+                             "is not compatible with Meson. (Can't locate artifacts in DUB's cache)."
+                             ' Upgrade to Dub >= 1.35')
             return
 
         mlog.debug('Determining dependency {!r} with DUB executable '
                    '{!r}'.format(name, self.dubbin.get_path()))
 
-        # if an explicit version spec was stated, use this when querying Dub
-        main_pack_spec = name
-        if 'version' in kwargs:
-            version_spec = kwargs['version']
-            if isinstance(version_spec, list):
-                version_spec = " ".join(version_spec)
-            main_pack_spec = f'{name}@{version_spec}'
-
         # we need to know the target architecture
         dub_arch = self.compiler.arch
 
         # we need to know the build type as well
-        dub_buildtype = str(environment.coredata.get_option(OptionKey('buildtype')))
+        dub_buildtype = str(environment.coredata.optstore.get_value_for(OptionKey('buildtype')))
         # MESON types: choices=['plain', 'debug', 'debugoptimized', 'release', 'minsize', 'custom'])),
         # DUB types: debug (default), plain, release, release-debug, release-nobounds, unittest, profile, profile-gc,
         # docs, ddox, cov, unittest-cov, syntax and custom
@@ -79,47 +130,29 @@ class DubDependency(ExternalDependency):
         elif dub_buildtype == 'minsize':
             dub_buildtype = 'release'
 
-        # Ask dub for the package
-        describe_cmd = [
-            'describe', main_pack_spec, '--arch=' + dub_arch,
-            '--build=' + dub_buildtype, '--compiler=' + self.compiler.get_exelist()[-1]
-        ]
-        ret, res, err = self._call_dubbin(describe_cmd)
-
-        if ret != 0:
-            mlog.debug('DUB describe failed: ' + err)
-            if 'locally' in err:
-                fetch_cmd = ['dub', 'fetch', main_pack_spec]
-                mlog.error(mlog.bold(main_pack_spec), 'is not present locally. You may try the following command:')
-                mlog.log(mlog.bold(join_args(fetch_cmd)))
-            self.is_found = False
+        result = self._get_dub_description(dub_arch, dub_buildtype)
+        if result is None:
             return
-
-        # A command that might be useful in case of missing DUB package
-        def dub_build_deep_command() -> str:
-            cmd = [
-                'dub', 'run', 'dub-build-deep', '--yes', '--', main_pack_spec,
-                '--arch=' + dub_arch, '--compiler=' + self.compiler.get_exelist()[-1],
-                '--build=' + dub_buildtype
-            ]
-            return join_args(cmd)
-
-        dub_comp_id = self.compiler.get_id().replace('llvm', 'ldc').replace('gcc', 'gdc')
-        description = json.loads(res)
+        description, build_cmd, description_source = result
+        dub_comp_id = self._ID_MAP[self.compiler.get_id()]
 
         self.compile_args = []
         self.link_args = self.raw_link_args = []
 
         show_buildtype_warning = False
 
-        def find_package_target(pkg: T.Dict[str, str]) -> bool:
+        # collect all targets
+        targets = {t['rootPackage']: t for t in description['targets']}
+
+        def find_package_target(pkg: DubPackDesc) -> bool:
             nonlocal show_buildtype_warning
             # try to find a static library in a DUB folder corresponding to
             # version, configuration, compiler, arch and build-type
             # if can find, add to link_args.
             # link_args order is meaningful, so this function MUST be called in the right order
             pack_id = f'{pkg["name"]}@{pkg["version"]}'
-            (tgt_file, compatibilities) = self._find_compatible_package_target(description, pkg, dub_comp_id)
+            tgt_desc = targets[pkg['name']]
+            (tgt_file, compatibilities) = self._find_target_in_cache(description, pkg, tgt_desc, dub_comp_id)
             if tgt_file is None:
                 if not compatibilities:
                     mlog.error(mlog.bold(pack_id), 'not found')
@@ -132,7 +165,7 @@ class DubDependency(ExternalDependency):
                     mlog.error(mlog.bold(pack_id), 'found but not compiled for', mlog.bold(dub_arch))
                 elif 'platform' not in compatibilities:
                     mlog.error(mlog.bold(pack_id), 'found but not compiled for',
-                               mlog.bold(description['platform'].join('.')))
+                               mlog.bold('.'.join(description['platform'])))
                 elif 'configuration' not in compatibilities:
                     mlog.error(mlog.bold(pack_id), 'found but not compiled for the',
                                mlog.bold(pkg['configuration']), 'configuration')
@@ -140,7 +173,7 @@ class DubDependency(ExternalDependency):
                     mlog.error(mlog.bold(pack_id), 'not found')
 
                 mlog.log('You may try the following command to install the necessary DUB libraries:')
-                mlog.log(mlog.bold(dub_build_deep_command()))
+                mlog.log(mlog.bold(build_cmd))
 
                 return False
 
@@ -159,38 +192,44 @@ class DubDependency(ExternalDependency):
         # 4. Add other build settings (imports, versions etc.)
 
         # 1
-        self.is_found = False
-        packages = {}
+        packages: T.Dict[str, DubPackDesc] = {}
+        found_it = False
         for pkg in description['packages']:
             packages[pkg['name']] = pkg
 
             if not pkg['active']:
                 continue
 
-            if pkg['targetType'] == 'dynamicLibrary':
-                mlog.error('DUB dynamic library dependencies are not supported.')
-                self.is_found = False
-                return
-
             # check that the main dependency is indeed a library
             if pkg['name'] == name:
-                self.is_found = True
-
                 if pkg['targetType'] not in ['library', 'sourceLibrary', 'staticLibrary']:
-                    mlog.error(mlog.bold(name), "found but it isn't a library")
-                    self.is_found = False
+                    mlog.error(mlog.bold(name), "found but it isn't a static library, it is:",
+                               pkg['targetType'])
                     return
 
+                if self.version_reqs is not None:
+                    ver = pkg['version']
+                    if not version_compare_many(ver, self.version_reqs)[0]:
+                        mlog.error(mlog.bold(f'{name}@{ver}'),
+                                   'does not satisfy all version requirements of:',
+                                   ' '.join(self.version_reqs))
+                        return
+
+                found_it = True
                 self.version = pkg['version']
                 self.pkg = pkg
 
-        # collect all targets
-        targets = {}
-        for tgt in description['targets']:
-            targets[tgt['rootPackage']] = tgt
+        if not found_it:
+            mlog.error(f'Could not find {name} in DUB description.')
+            if description_source is DubDescriptionSource.Local:
+                mlog.log('Make sure that the dependency is registered for your dub project by running:')
+                mlog.log(mlog.bold(f'dub add {name}'))
+            elif description_source is DubDescriptionSource.External:
+                # `dub describe pkg` did not contain the pkg
+                raise RuntimeError(f'`dub describe` succeeded but it does not contains {name}')
+            return
 
         if name not in targets:
-            self.is_found = False
             if self.pkg['targetType'] == 'sourceLibrary':
                 # source libraries have no associated targets,
                 # but some build settings like import folders must be found from the package object.
@@ -199,10 +238,7 @@ class DubDependency(ExternalDependency):
                 # (See openssl DUB package for example of sourceLibrary)
                 mlog.error('DUB targets of type', mlog.bold('sourceLibrary'), 'are not supported.')
             else:
-                mlog.error('Could not find target description for', mlog.bold(main_pack_spec))
-
-        if not self.is_found:
-            mlog.error(f'Could not find {name} in DUB description')
+                mlog.error('Could not find target description for', mlog.bold(self.name))
             return
 
         # Current impl only supports static libraries
@@ -210,19 +246,17 @@ class DubDependency(ExternalDependency):
 
         # 2
         if not find_package_target(self.pkg):
-            self.is_found = False
             return
 
         # 3
         for link_dep in targets[name]['linkDependencies']:
             pkg = packages[link_dep]
             if not find_package_target(pkg):
-                self.is_found = False
                 return
 
         if show_buildtype_warning:
             mlog.log('If it is not suitable, try the following command and reconfigure Meson with', mlog.bold('--clearcache'))
-            mlog.log(mlog.bold(dub_build_deep_command()))
+            mlog.log(mlog.bold(build_cmd))
 
         # 4
         bs = targets[name]['buildSettings']
@@ -269,7 +303,7 @@ class DubDependency(ExternalDependency):
         for lib in bs['libs']:
             if os.name != 'nt':
                 # trying to add system libraries by pkg-config
-                pkgdep = PkgConfigDependency(lib, environment, {'required': 'true', 'silent': 'true'})
+                pkgdep = PkgConfigDependency(lib, environment, {'required': True, 'silent': True})
                 if pkgdep.is_found:
                     for arg in pkgdep.get_compile_args():
                         self.compile_args.append(arg)
@@ -286,17 +320,80 @@ class DubDependency(ExternalDependency):
             # fallback
             self.link_args.append('-l'+lib)
 
+        self.is_found = True
+
+    # Get the dub description needed to resolve the dependency and a
+    # build command that can be used to build the dependency in case it is
+    # not present.
+    def _get_dub_description(self, dub_arch: str, dub_buildtype: str) -> T.Optional[T.Tuple[DubDescription, str, DubDescriptionSource]]:
+        def get_build_command() -> T.List[str]:
+            if self._dub_has_build_deep:
+                cmd = ['dub', 'build', '--deep']
+            else:
+                cmd = ['dub', 'run', '--yes', 'dub-build-deep', '--']
+
+            return cmd + [
+                '--arch=' + dub_arch,
+                '--compiler=' + self.compiler.get_exelist()[-1],
+                '--build=' + dub_buildtype,
+            ]
+
+        # Ask dub for the package
+        describe_cmd = [
+            'describe', '--arch=' + dub_arch,
+            '--build=' + dub_buildtype, '--compiler=' + self.compiler.get_exelist()[-1]
+        ]
+        helper_build = join_args(get_build_command())
+        source = DubDescriptionSource.Local
+        ret, res, err = self._call_dubbin(describe_cmd)
+        if ret == 0:
+            return (json.loads(res), helper_build, source)
+
+        pack_spec = self.name
+        if self.version_reqs is not None:
+            if len(self.version_reqs) > 1:
+                mlog.error('Multiple version requirements are not supported for raw dub dependencies.')
+                mlog.error("Please specify only an exact version like '1.2.3'")
+                raise DependencyException('Multiple version requirements are not solvable for raw dub depencies')
+            elif len(self.version_reqs) == 1:
+                pack_spec += '@' + self.version_reqs[0]
+
+        describe_cmd = [
+            'describe', pack_spec, '--arch=' + dub_arch,
+            '--build=' + dub_buildtype, '--compiler=' + self.compiler.get_exelist()[-1]
+        ]
+        helper_build = join_args(get_build_command() + [pack_spec])
+        source = DubDescriptionSource.External
+        ret, res, err = self._call_dubbin(describe_cmd)
+        if ret == 0:
+            return (json.loads(res), helper_build, source)
+
+        mlog.debug('DUB describe failed: ' + err)
+        if 'locally' in err:
+            mlog.error(mlog.bold(pack_spec), 'is not present locally. You may try the following command:')
+            mlog.log(mlog.bold(helper_build))
+        return None
+
     # This function finds the target of the provided JSON package, built for the right
     # compiler, architecture, configuration...
     # It returns (target|None, {compatibilities})
     # If None is returned for target, compatibilities will list what other targets were found without full compatibility
-    def _find_compatible_package_target(self, jdesc: T.Dict[str, str], jpack: T.Dict[str, str], dub_comp_id: str) -> T.Tuple[str, T.Set[str]]:
-        dub_build_path = os.path.join(jpack['path'], '.dub', 'build')
+    def _find_target_in_cache(self, desc: DubDescription, pkg_desc: DubPackDesc,
+                              tgt_desc: DubTargetDesc, dub_comp_id: str
+                              ) -> T.Tuple[T.Optional[str], T.Set[str]]:
+        mlog.debug('Searching in DUB cache for compatible', pkg_desc['targetFileName'])
 
-        if not os.path.exists(dub_build_path):
-            return (None, None)
+        # recent DUB versions include a direct path to a compatible cached artifact
+        if self._use_cache_describe:
+            tgt_file = tgt_desc['cacheArtifactPath']
+            if os.path.exists(tgt_file):
+                return (tgt_file, {'configuration', 'platform', 'arch', 'compiler', 'compiler_version', 'build_type'})
+            else:
+                return (None, set())
 
-        # try to find a dir like library-debug-linux.posix-x86_64-ldc_2081-EF934983A3319F8F8FF2F0E107A363BA
+        assert self._search_in_cache
+
+        # try to find a string like library-debug-linux.posix-x86_64-ldc_2081-EF934983A3319F8F8FF2F0E107A363BA
 
         # fields are:
         #  - configuration
@@ -306,40 +403,16 @@ class DubDependency(ExternalDependency):
         #  - compiler id (dmd, ldc, gdc)
         #  - compiler version or frontend id or frontend version?
 
-        conf = jpack['configuration']
-        build_type = jdesc['buildType']
-        platforms = jdesc['platform']
-        archs = jdesc['architecture']
-
-        # Get D frontend version implemented in the compiler, or the compiler version itself
-        # gdc doesn't support this
-        comp_versions = []
-
-        if dub_comp_id != 'gdc':
-            comp_versions.append(self.compiler.version)
-
-            ret, res = self._call_compbin(['--version'])[0:2]
-            if ret != 0:
-                mlog.error('Failed to run {!r}', mlog.bold(dub_comp_id))
-                return (None, None)
-            d_ver_reg = re.search('v[0-9].[0-9][0-9][0-9].[0-9]', res)  # Ex.: v2.081.2
-
-            if d_ver_reg is not None:
-                frontend_version = d_ver_reg.group()
-                frontend_id = frontend_version.rsplit('.', 1)[0].replace(
-                    'v', '').replace('.', '')  # Fix structure. Ex.: 2081
-                comp_versions.extend([frontend_version, frontend_id])
-
-        compatibilities: T.Set[str] = set()
+        comp_versions = self._get_comp_versions_to_find(dub_comp_id)
 
         # build_type is not in check_list because different build types might be compatible.
         # We do show a WARNING that the build type is not the same.
         # It might be critical in release builds, and acceptable otherwise
-        check_list = ('configuration', 'platform', 'arch', 'compiler', 'compiler_version')
+        check_list = {'configuration', 'platform', 'arch', 'compiler', 'compiler_version'}
+        compatibilities: T.Set[str] = set()
 
-        for entry in os.listdir(dub_build_path):
-
-            target = os.path.join(dub_build_path, entry, jpack['targetFileName'])
+        for entry in self._cache_entries(pkg_desc):
+            target = entry['artifactPath']
             if not os.path.exists(target):
                 # unless Dub and Meson are racing, the target file should be present
                 # when the directory is present
@@ -349,36 +422,83 @@ class DubDependency(ExternalDependency):
             # we build a new set for each entry, because if this target is returned
             # we want to return only the compatibilities associated to this target
             # otherwise we could miss the WARNING about build_type
-            comps = set()
+            comps: T.Set[str] = set()
 
-            if conf in entry:
+            search = entry['search']
+
+            mlog.debug('searching compatibility in ' + search)
+            mlog.debug('compiler_versions', comp_versions)
+
+            if pkg_desc['configuration'] in search:
                 comps.add('configuration')
 
-            if build_type in entry:
+            if desc['buildType'] in search:
                 comps.add('build_type')
 
-            if all(platform in entry for platform in platforms):
+            if all(platform in search for platform in desc['platform']):
                 comps.add('platform')
 
-            if all(arch in entry for arch in archs):
+            if all(arch in search for arch in desc['architecture']):
                 comps.add('arch')
 
-            if dub_comp_id in entry:
+            if dub_comp_id in search:
                 comps.add('compiler')
 
-            if dub_comp_id == 'gdc' or any(cv in entry for cv in comp_versions):
+            if not comp_versions or any(cv in search for cv in comp_versions):
                 comps.add('compiler_version')
 
-            if all(key in comps for key in check_list):
+            if check_list.issubset(comps):
+                mlog.debug('Found', target)
                 return (target, comps)
             else:
                 compatibilities = set.union(compatibilities, comps)
 
         return (None, compatibilities)
 
+    def _cache_entries(self, pkg_desc: DubPackDesc) -> T.List[FindTargetEntry]:
+        # the "old" cache is the `.dub` directory in every package of ~/.dub/packages
+        dub_build_path = os.path.join(pkg_desc['path'], '.dub', 'build')
+
+        if not os.path.exists(dub_build_path):
+            mlog.warning('No such cache folder:', dub_build_path)
+            return []
+
+        mlog.debug('Checking in DUB cache folder', dub_build_path)
+
+        return [
+            {
+                'search': dir_entry,
+                'artifactPath': os.path.join(dub_build_path, dir_entry, pkg_desc['targetFileName'])
+            }
+            for dir_entry in os.listdir(dub_build_path)
+        ]
+
+    def _get_comp_versions_to_find(self, dub_comp_id: str) -> T.List[str]:
+        # Get D frontend version implemented in the compiler, or the compiler version itself
+        # gdc doesn't support this
+
+        if dub_comp_id == 'gdc':
+            return []
+
+        comp_versions = [self.compiler.version]
+
+        ret, res = self._call_compbin(['--version'])[0:2]
+        if ret != 0:
+            mlog.error('Failed to run', mlog.bold(' '.join(self.dubbin.get_command() + ['--version'])))
+            return []
+        d_ver_reg = re.search('v[0-9].[0-9][0-9][0-9].[0-9]', res)  # Ex.: v2.081.2
+
+        if d_ver_reg is not None:
+            frontend_version = d_ver_reg.group()
+            frontend_id = frontend_version.rsplit('.', 1)[0].replace(
+                'v', '').replace('.', '')  # Fix structure. Ex.: 2081
+            comp_versions.extend([frontend_version, frontend_id])
+
+        return comp_versions
+
     def _call_dubbin(self, args: T.List[str], env: T.Optional[T.Dict[str, str]] = None) -> T.Tuple[int, str, str]:
         assert isinstance(self.dubbin, ExternalProgram)
-        p, out, err = Popen_safe(self.dubbin.get_command() + args, env=env)
+        p, out, err = Popen_safe(self.dubbin.get_command() + args, env=env, cwd=self.env.get_source_dir())
         return p.returncode, out.strip(), err.strip()
 
     def _call_compbin(self, args: T.List[str], env: T.Optional[T.Dict[str, str]] = None) -> T.Tuple[int, str, str]:
