@@ -1,3 +1,3 @@
-int func4() {
+int func4(void) {
     return 4;
 }
