@@ -1,10 +1,9 @@
 #import<Foundation/NSString.h>
 
-int main(int argc, char **argv) {
+int main(void) {
   int result;
   NSString *str = [NSString new];
   result = [str length];
   [str release];
   return result;
 }
-
