@@ -1,5 +1,5 @@
 #include "vis.h"
 
-int EXPORT_PUBLIC libfunc() {
+int EXPORT_PUBLIC libfunc(void) {
     return 3;
 }
