@@ -1,19 +1,7 @@
 #!/usr/bin/env python3
-
+# SPDX-License-Identifier: Apache-2.0
 # Copyright 2015 The Meson development team
 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-
-#     http://www.apache.org/licenses/LICENSE-2.0
-
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 help_message = """Usage: {} <config.h.meson>
 
 This script reads config.h.meson, looks for header
@@ -382,7 +370,7 @@ with open(sys.argv[1], encoding='utf-8') as f:
             tarr = token.split('_')[1:-1]
             tarr = [x.lower() for x in tarr]
             hname = '/'.join(tarr) + '.h'
-            headers.append((token, hname))
+            headers.append(hname)
 
         # Check for functions.
         try:
@@ -413,13 +401,13 @@ cdata = configuration_data()''')
 # Convert header checks.
 
 print('check_headers = [')
-for token, hname in headers:
-    print(f"  ['{token}', '{hname}'],")
+for hname in headers:
+    print(f"  '{hname}',")
 print(']\n')
 
 print('''foreach h : check_headers
-  if cc.has_header(h.get(1))
-    cdata.set(h.get(0), 1)
+  if cc.has_header(h)
+    cdata.set('HAVE_' + h.underscorify().to_upper(), 1)
   endif
 endforeach
 ''')
