@@ -95,6 +95,23 @@ class PlatformAgnosticTests(BasePlatformTests):
         testdir = os.path.join(self.unit_test_dir, '102 python without pkgconfig')
         self.init(testdir, override_envvars={'PKG_CONFIG': 'notfound'})
 
+    def test_vala_target_with_internal_glib(self):
+        testdir = os.path.join(self.unit_test_dir, '130 vala internal glib')
+        for run in [{ 'version': '2.84.4', 'expected': '2.84'}, { 'version': '2.85.2', 'expected': '2.84' }]:
+            self.new_builddir()
+            self.init(testdir, extra_args=[f'-Dglib-version={run["version"]}'])
+            try:
+                with open(os.path.join(self.builddir, 'meson-info', 'intro-targets.json'), 'r', encoding='utf-8') as tgt_intro:
+                    intro = json.load(tgt_intro)
+                    target = list(filter(lambda tgt: tgt['name'] == 'vala-tgt', intro))
+                    self.assertLength(target, 1)
+                    sources = target[0]['target_sources']
+                    vala_sources = filter(lambda src: src.get('language') == 'vala', sources)
+                    for src in vala_sources:
+                        self.assertIn(('--target-glib', run['expected']), zip(src['parameters'], src['parameters'][1:]))
+            except FileNotFoundError:
+                self.skipTest('Current backend does not produce introspection data')
+
     def test_debug_function_outputs_to_meson_log(self):
         testdir = os.path.join(self.unit_test_dir, '104 debug function')
         log_msg = 'This is an example debug output, should only end up in debug log'
